/* the cantus project.
 * (c)2002 by Samuel Abels (sam@manicsadness.com)
 * This project's homepage is: http://software.manicsadness.com/cantus
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <libgnomeui/gnome-messagebox.h>
      

#include "freedb.h"
#include "lib_freedb.h"
#include "gui_freedb.h"
#include "support.h"
#include "interface.h"
#include "mp3info.h"
#include "gui_queue.h"
#include "lib_id3v1.h"
#include "lib_id3v2.3.h"
#include "lib_vorbis.h"
#include "configfile.h"
#include "plugs.h"
#include "shared.h"
#include "gui_shared.h"


/***************************************************************************************
 * BELOW FOLLOW THE STATICS
 ***************************************************************************************/

// Returns the datapointer to the queueitems data with the given fullfilename
static Mp3Info *
g_list_find_matching_fullfilename(GList *haystack, gchar *fullfilename)
{
	GList *haystack_item = NULL;
	Mp3Info *haystack_mp3info = NULL;
	gchar haystack_fullfilename[2048];

	haystack_item = g_list_first(haystack);
	while(haystack_item)
	{
		haystack_mp3info = (Mp3Info *)haystack_item->data;
		haystack_item = g_list_next(haystack_item);
		
		snprintf(haystack_fullfilename, 2047, "%s%s", haystack_mp3info->directory, haystack_mp3info->filename);
		if(strncmp(fullfilename, haystack_fullfilename, 2048)==0)
			return haystack_mp3info;
	}
	return NULL;
}


// Check if the user entered all data correctly.
static gboolean
check_userinput(void)
{
	extern GtkWidget *cantus;
	
	if( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget(cantus, "radio_cantus_freedb_fulltext"))) )
	{
		if( *gtk_entry_get_text(GTK_ENTRY(lookup_widget(cantus, "entry_cantus_freedb_fulltext_string"))) == '\0' )
		{
			popup_info("Please enter a search string!");
			return(FALSE);
		}
	}
	else
	{
		if( *gtk_entry_get_text(GTK_ENTRY(lookup_widget(cantus, "entry_cantus_freedb_id_category"))) == '\0' )
		{
			popup_info("You must enter a FreeDB category!");
			return(FALSE);
		}
		if( *gtk_entry_get_text(GTK_ENTRY(lookup_widget(cantus, "entry_cantus_freedb_id_id"))) == '\0' )
		{
			popup_info("You must enter a FreeDB ID!");
			return(FALSE);
		}
	}
	return(TRUE);
}

static void
get_filenames_from_queue( CDInfo *cd )
{
	extern GtkWidget *cantus;
	CDTrack *track = NULL;
	Mp3Info *mp3info = NULL;
	gint row = 0;
	
	if( !cd )
		return;
	
	track = cd->track[0];
	while( (mp3info = gtk_clist_get_row_data(GTK_CLIST(lookup_widget(cantus, "clist_cantus_queue")), row)) != NULL )	
	{
		if( track == NULL )
			break;
		
		snprintf( track->filename, 2047, "%s%s", mp3info->directory, mp3info->filename );
		
		track = cd->track[++row];
		if( track )
			*(track->filename) = '\0';
	}
	
	return;
}

/***************************************************************************************
 * END OF STATICS
 ***************************************************************************************/






void
freedb_get(void)
{
	extern GtkWidget *cantus;
	extern GtkWidget *tracks_sort;

	gchar *host_name[50];
	guint host_port[50];
	gchar *cgi_path[50];
	gchar proxy_name[2048];
	guint proxy_port = 0;

	GList *cdlist = NULL;
	
	gchar option[2048];
	gchar value[2048];
	gint cur = 0;
	gchar config[10000];
	
	if( !check_userinput() )
		return;
	
// read configfile
	config_read(config, 10000);

// Get list of available FreeDB servers.
	host_name[0] = NULL;
	snprintf(option, 2047, "server0#1");
	while(configfile_option_get(config, "freedb servers", option, value))
	{
		host_name[cur] = strdup(value);
		
		snprintf(option, 2047, "server0#2");
		configfile_option_get(config, "freedb servers", option, value);
		host_port[cur] = (guint)atoi(value);
		
		snprintf(option, 2047, "server0#3");
		configfile_option_get(config, "freedb servers", option, value);

		snprintf(option, 2047, "server0#4");
		configfile_option_get(config, "freedb servers", option, value);
		cgi_path[cur] = strdup(value);
		
		host_name[++cur] = NULL;
		snprintf(option, 2047, "server%i#1", cur);
	}
// get the proxy server
	configfile_option_get(config, "freedb", "proxy address", proxy_name);
	configfile_option_get(config, "freedb", "proxy port", value);
	proxy_port = (guint)atoi(value);
	configfile_option_get(config, "freedb", "enable proxy", value);
	if(strcmp(value, "TRUE")!=0)
		*proxy_name = '\0';
	
	if( host_name[0] == NULL )
		return;
	
// get either by freedb id, or by a searchstring.
	if( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget(cantus, "radio_cantus_freedb_fulltext"))) )
	{
		gint socket = 0;
		
// connect to freedb host
		if( (socket = freedb_connect_host("www.freedb.org", 80, proxy_name, proxy_port)) < 0 )
			return;
		
// Get album list.
		if( freedb_get_albumlist_by_searchstring(
			socket,
			gtk_entry_get_text(GTK_ENTRY(lookup_widget(cantus, "entry_cantus_freedb_fulltext_string"))),
			&cdlist) < 0 )
		{
			close(socket);
			return;
		}

// Close connection.
		close(socket);
		
// show gui and insert the data
		tracks_sort = create_tracks_sort();
		clist_album_choose_albums_add(cdlist);
		gtk_widget_show(tracks_sort);
		g_list_free(cdlist);
	}
	else
	{
		gint socket = 0;
		CDInfo *cd = calloc(1, sizeof(CDInfo));
		
// get genre and id from the gui.
		strncpy (cd->genre, gtk_entry_get_text(GTK_ENTRY(lookup_widget(cantus, "entry_cantus_freedb_id_category"))), 512);
		cd->id = strtoul (gtk_entry_get_text(GTK_ENTRY(lookup_widget(cantus, "entry_cantus_freedb_id_id"))), 0, 16);

// Connect to any host.
		cur = -1;
		while( socket <= 0
			&& host_name[++cur] != NULL )
			socket = freedb_connect_host (host_name[cur], host_port[cur], proxy_name, proxy_port);
		
// Still not connected? Then return.
		if( socket <= 0 )
			return;

// Get album from Freedb.
		if( freedb_get_album_by_id_and_category (socket, host_name[cur], host_port[cur], cgi_path[cur], "cantus", VERSION, &cd) != 0 )
		{
			close(socket);
			return;
		}
		
// Close connection.
		close(socket);
		
// Add the CURRENT filenames to the struct, also.
		get_filenames_from_queue(cd);
		
// Show the track_sort widget and insert the albums data (=Tracks/Titles).
		tracks_sort = create_tracks_sort();
		clist_tracks_sort_tracks_add(cd);
		gtk_widget_show(tracks_sort);
	}
	
// cleanups
	cur = -1;
	while(host_name[++cur])
	{
		g_free(host_name[cur]);
		g_free(cgi_path[cur]);
	}
}



void
freedb_album_clicked(void)
{
	extern GtkWidget *tracks_sort;
	gint row = 0;
	
	CDInfo *cd = NULL;
	
	gchar *host_name[50];
	guint host_port[50];
	gchar *cgi_path[50];
	gchar proxy_name[2048];
	guint proxy_port = 0;

	gchar option[2048];
	gchar value[2048];
	gint cur = 0;
	gchar config[10000];
	gint socket = 0;
	
// Get the chosen albums data
	row = (gint)(GTK_CLIST(lookup_widget(tracks_sort, "clist_tracks_sort_cdlist"))->selection)->data;
	cd = (CDInfo *)gtk_clist_get_row_data(GTK_CLIST(lookup_widget(tracks_sort, "clist_tracks_sort_cdlist")), row);
	
	if( cd == NULL )
		return;
	
// There may already be filenames in the cd struct. Remove them.
	cur = -1;
	while( cd->track[++cur] != NULL )
	{
		free(cd->track[cur]);
		cd->track[cur] = NULL;
	}
	
// read the configfile
	config_read(config, 10000);

// Get list of available FreeDB servers.
	cur = 0;
	host_name[cur] = NULL;
	snprintf(option, 2047, "server0#1");
	while( configfile_option_get(config, "freedb servers", option, value) )
	{
		host_name[cur] = strdup(value);
		
		snprintf(option, 2047, "server0#2");
		configfile_option_get(config, "freedb servers", option, value);
		host_port[cur] = (guint)atoi(value);
		
		snprintf(option, 2047, "server0#3");
		configfile_option_get(config, "freedb servers", option, value);

		snprintf(option, 2047, "server0#4");
		configfile_option_get(config, "freedb servers", option, value);
		cgi_path[cur] = strdup(value);
		
		host_name[++cur] = NULL;
		snprintf(option, 2047, "server%i#1", cur);
	}
// get the proxy server
	configfile_option_get(config, "freedb", "proxy address", proxy_name);
	configfile_option_get(config, "freedb", "proxy port", value);
	proxy_port = (guint)atoi(value);
	configfile_option_get(config, "freedb", "enable proxy", value);
	if( strcmp(value, "TRUE") != 0 )
		*proxy_name = '\0';

// no hosts? return.
	if( host_name[0] == NULL )
		goto cleanups;

// if none is selected, return.
	if( !(GTK_CLIST(lookup_widget(tracks_sort, "clist_tracks_sort_cdlist"))->selection) )
		goto cleanups;
	
// Connect to any host.
	cur = -1;
	while( socket <= 0
		&& host_name[++cur] != NULL )
		socket = freedb_connect_host(host_name[cur], host_port[cur], proxy_name, proxy_port);
	
// Still not connected? Then return.
	if( socket <= 0 )
		goto cleanups;

// Get album from Freedb.
	if( freedb_get_album_by_id_and_category(socket, host_name[cur], host_port[cur], cgi_path[cur], "cantus", VERSION, &cd) != 0 )
	{
		close(socket);
		goto cleanups;
	}
	
// Close connection.
	close(socket);
	
// Add the CURRENT filenames to the struct, also.
	get_filenames_from_queue(cd);
	
// Show the track_sort widget and insert the albums data (=Tracks/Titles).
	clist_tracks_sort_tracks_add(cd);
	
// cleanups
cleanups:
	cur = -1;
	while(host_name[++cur])
	{
		g_free(host_name[cur]);
		g_free(cgi_path[cur]);
	}
}




void
freedb_tracks_ok(void)
{
	extern GtkWidget *cantus;
	extern GtkWidget *tracks_sort;
	extern GList *queue;
	extern gchar *active_tag;
	
	gchar newfilename[2048];
	gchar newfilename_full[2048];
	CDInfo *cd = NULL;
	CDTrack *curtrack = NULL;
	gint row = -1, num = -1;
	gint whichtag = 0;

	id3Tag *tag = calloc(1, sizeof(id3Tag));
	Mp3Info *mp3info = NULL;
	
	cd = (CDInfo *)gtk_clist_get_row_data(GTK_CLIST(lookup_widget(tracks_sort, "clist_tracks_sort_tracks")), 0);
	
	if( cd == NULL )
		goto cleanups;

	row = -1;
	while(cd->track[++row])
	{
		curtrack = (CDTrack *)cd->track[row];
		
		if( *(curtrack->filename) != '\0' )
		{
			mp3info = g_list_find_matching_fullfilename(queue, curtrack->filename);
			
			memset(tag->artist, '\0', 1024);
			memset(tag->title, '\0', 1024);
			memset(tag->album, '\0', 1024);
			memset(tag->year, '\0', 5);
			memset(tag->track, '\0', 3);
			memset(tag->comment, '\0', 1024);
			memset(tag->genre, '\0', 512);

// Get old Tag
			if( strtoucmp(curtrack->filename + strlen(curtrack->filename)-4, ".mp3") == 0 )
			{
				id3Tag *v2 = malloc(sizeof(id3Tag));
				
// get the tag to a struct.
				get_id3v1_tag(tag, curtrack->filename);

// v2 tag has higher prio, so it will overwrite the v1	
				if( get_id3v2_tag(v2, curtrack->filename) == 0 )
				{
					g_free(tag);
					tag = v2;
				}
				else
					g_free(v2);
				
				whichtag = 1;
			}
#ifdef HAVE_OGG_H
			else
			{
				get_vorbis_tag(tag, curtrack->filename);
				whichtag = 3;
			}
#endif

			
// And put the changed tag to the file
			if( access(curtrack->filename, W_OK) == 0 )
			{
// the cdtitle can have two different formats in freedb:
// either "Artist / Album" or only "Sampler" or stuff.
// so we look for an " / ", if its there, we split it into artist and cdtitle
// otherwise we set both the same, artist and cdtitle.
// copy freedb album title to tag
				if( strstr(cd->cdtitle, " / ") != NULL )
				{
					strncpy(tag->artist, cd->cdtitle, 1024);
					*(strstr(tag->artist, " / ")) = '\0';
					strncpy(tag->album, strstr(cd->cdtitle, " / ")+3, 1024);
				}
				else
				{
					strncpy(tag->artist, cd->cdtitle, 1024);
					strncpy(tag->album, cd->cdtitle, 1024);
				}

// copy track title to tag
// all the same as the cdtitle, we have the two possible formats.
				if( strstr(curtrack->title, " / ") != NULL )
				{
					strncpy(tag->artist, curtrack->title, 1024);
					*(strstr(tag->artist, " / ")) = '\0';
					strncpy(tag->title, strstr(curtrack->title, " / ")+3, 1024);
				}
				else
					strncpy(tag->title, curtrack->title, 1024);
				
// track number and year.
				snprintf(tag->track, 3, "%s", curtrack->number);
				strncpy(tag->year, cd->year, 5);

// save tags.
				if( whichtag == 1
					&& *curtrack->filename != '\0' )
				{
					set_id3v1_tag(tag, curtrack->filename);
				
					if( set_id3v2_tag(tag, curtrack->filename) == 0 )
					{
// after we set the tag, we re-read its tag information.
						mp3file_get_info_1(mp3info->directory, mp3info->filename, mp3info);
// I then create the new filename.
						snprintf(newfilename, 2047, "%s - %s.mp3", curtrack->number, curtrack->title);
						apply_rule_replace_invalid_characters (newfilename);
// and the new fullfilename
						strncpy(newfilename_full, curtrack->filename, 2047);
						*(strrchr(newfilename_full, '/') + 1) = '\0';
						strncpy( newfilename_full + strlen(newfilename_full), newfilename, 2047 - strlen(newfilename_full) );
// just to be sure we do nothing stupid we replace a " / " with a " - "
						if ( strstr(newfilename_full, " / ") != NULL )
							*(strstr(newfilename_full, " / ") + 1) = '-';

// be sure we don't overwrite any old file
						if( *curtrack->filename != '\0'
							&& access(newfilename_full, F_OK)==0 )
						{
							strncpy(mp3info->status, "File exists!", 255);
						}
						else
						{
// Rename file
							if(rename(curtrack->filename, newfilename_full)==0)
							{
								strncpy(mp3info->filename, newfilename, 2047);
								strncpy(mp3info->status, "FreeDB OK!", 255);
								if(strcmp(active_tag, curtrack->filename) == 0)
									strncpy(active_tag, newfilename_full, 2047);
							}
							else
							{
								strncpy(mp3info->status, "Tags set, rename failed!", 255);
							}
						}
					}
					else
						strncpy(mp3info->status, "Failed!", 255);
				}
#ifdef HAVE_OGG_H
// The same for Ogg.
				else
				{
					if(set_vorbis_tag(tag, curtrack->filename) == 0)
					{
// after we set the tag, we re-read its tag information.
						mp3file_get_info_1(mp3info->directory, mp3info->filename, mp3info);
// I then create the new filename.
						snprintf(newfilename, 2047, "%s - %s.ogg", curtrack->number, curtrack->title);
						apply_rule_replace_invalid_characters(newfilename);
// and the new fullfilename
						strncpy(newfilename_full, curtrack->filename, 2047);
						*(strrchr(newfilename_full, '/')+1) = '\0';
						strncpy( newfilename_full+strlen(newfilename_full), newfilename, 2047-strlen(newfilename_full) );
// just to be sure we do nothing stupid we replace a " / " with a " - "
						if(strstr(newfilename_full, " / ") != NULL)
							*(strstr(newfilename_full, " / ") + 1) = '-';

// be sure we don't overwrite any old file
						if( *curtrack->filename != '\0'
							&& access(newfilename_full, F_OK) == 0 )
						{
							strncpy(mp3info->status, "File exists!", 255);
						}
						else
						{
// Rename file
							if(rename(curtrack->filename, newfilename_full) == 0)
							{
								strncpy(mp3info->filename, newfilename, 2047);
								strncpy(mp3info->status, "FreeDB OK!", 255);
								if(strcmp(active_tag, curtrack->filename) == 0)
									strncpy(active_tag, newfilename_full, 2047);
							}
							else
								strncpy(mp3info->status, "Tags set, rename failed!", 255);
						}
					}
					else
						strncpy(mp3info->status, "Failed!", 255);
				}
#endif
			}
			else
			{
// ok, we don't have write access
				if( *curtrack->filename != '\0'
					&& access(curtrack->filename, W_OK) != 0 )
				{
					strncpy(mp3info->status, "Locked!", 255);
				}
				else
					strncpy(mp3info->status, "Unchanged!", 255);
			}
		}
		clist_file_update(GTK_CLIST(lookup_widget(cantus, "clist_cantus_queue")), mp3info);
		clist_file_update(GTK_CLIST(lookup_widget(cantus, "clist_cantus_queueonly_queue")), mp3info);
		
		g_free(curtrack);
		cd->track[row] = NULL;
	}

// cleanups
cleanups:
	g_free(tag);
	
// free every cd in the cd list.
	row = -1;
	num = -1;
	while( (cd = gtk_clist_get_row_data(GTK_CLIST(lookup_widget(tracks_sort, "clist_tracks_sort_cdlist")), ++row)) )
	{
		if( cd != NULL )
		{
			num = -1;
			while( cd->track[++num] )
				g_free(cd->track[num]);
		}
		
		free(cd);
	}
}





void
freedb_tracks_cancel(void)
{
	extern GtkWidget *tracks_sort;
	CDInfo *cd = NULL;
	gint row = -1, num = -1;

// free every cd in the cd list.
	while( (cd = gtk_clist_get_row_data(GTK_CLIST(lookup_widget(tracks_sort, "clist_tracks_sort_cdlist")), ++row)) )
	{
		if( cd != NULL )
		{
			num = -1;
			while( cd->track[++num] )
				g_free(cd->track[num]);
		}
		
		free(cd);
	}
	
	gtk_widget_destroy(tracks_sort);
}
