/* the cantus project.
 * (c)2002 by Samuel Abels (sam@manicsadness.com)
 * This project's homepage is: http://software.manicsadness.com/cantus
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include <gnome.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include "lib_freedb.h"
#include "plugs.h"


/***************************************************************************************
 * BELOW FOLLOW THE STATICS
 ***************************************************************************************/

// receive one line from a host
static gint
get_line(gint socket, gchar *line, gint maxlen)
{
	gint i = 0;
	gint foo = 0;
	
// We need to get char by char, to the first occurence of a line feed, so we get only one line.
	memset(line, 0, maxlen);
	for(i=0; i < maxlen; i++)
	{
		if( strlen(line) > 2
			&& *(line + i - 1) == '\n' )
			break;
		foo = read(socket, line + i, 1);
	}
	
// remove the linefeed.
	while( (*(line + strlen(line) - 1) == '\n' || *(line + strlen(line) - 1) == '\r')
		&& strlen(line) > 0 )
		*(line + strlen(line) - 1) = 0;
	
	return foo;
}

/***************************************************************************************
 * END OF STATICS
 ***************************************************************************************/



// Connect to a host, return the socket number.
gint
freedb_connect_host(gchar *hostname, guint host_port, gchar *proxy, guint proxy_port)
{
	struct sockaddr_in socket_address;
	struct hostent *hostent;
	gint socket_number;

// Get a Socket number
	if((socket_number = socket(AF_INET, SOCK_STREAM, 0)) == -1)
	{
		perror("Error: Can't open INET socket!");
		return -1;
	}
	
// No proxy? Then the host is proxy, also.
	if(!proxy
		|| *proxy=='\0')
	{
		proxy = hostname;
		proxy_port = host_port;
	}
	
// Server address lookup
	if( !(hostent = gethostbyname(proxy)) )
	{
		perror("Failed to lookup the server!");
		return -2;
	}
	
	socket_address.sin_family = AF_INET;
	memcpy(&socket_address.sin_addr, hostent->h_addr, hostent->h_length);
	socket_address.sin_port = htons(proxy_port);

// Connect to the server...
	if( connect(socket_number, (struct sockaddr *)&socket_address, sizeof(socket_address)) == -1 )
	{
		perror("Can't connect to FreeDB host!");
		return -3;
	}
	
	return(socket_number);
}




gint
freedb_get_album_by_id_and_category(gint socket_number, gchar *hostname, guint host_port, gchar *cgi_path, gchar *progname, gchar *version, CDInfo **cd)
{
	gchar bin[4096] = "";		// Binary in data
	gchar bout[4096] = "";		// Binary out data
	gchar *myusername = getenv("USER");
	gchar myhostname[2048] = "";

// A socket > 0 is mandatory.
	if( socket_number <= 0 )
		return(-1);
	
// Get our hostname
    gethostname(myhostname, sizeof(myhostname)-1);
	
// Create request
	snprintf( bout, 4095, "GET http://%s%s?cmd=cddb+read+%s+%08lx&hello=%s@%s+%s+%s+%s&proto=1 HTTP/1.1\r\nConnection: close\nHost: %s:%i\r\n\r\n",
		hostname,
		cgi_path,
		(*cd)->genre,
		(*cd)->id,
		myusername,
		myhostname,
		myhostname,
		progname,
		version,
		hostname,
		host_port );
	
// Send request
	if( write(socket_number, bout, strlen(bout)) < 0 )
	{
		perror("Error while sending request to the server!");
		return(-2);
	}

// We get the answer of the query
// the first line does NOT contain a title (and there may be more lines like that), but some header informations
// we dont care about.
	if( get_line(socket_number, bin, sizeof(bin)) <= 0 )
	{
		perror("Error while receiving request from the server!");
		return(-3);
	}
	if( strncmp(bin, "HTTP/1.1 200 ", 13) != 0 )
	{
		printf("Error while receiving track information. (%s)\n", bin);
		return(-4);
	}

// so the server sent a valid reply, we can get all titles now.
// we only care about lines beginning with "TTITLE", that will skip all header lines and stuff.
// a line containing nothing but a dot (plus a linefeed) means end of data.
	(*cd)->track[0] = NULL;
	while( strcmp(bin, ".\r\n") != 0 )
	{
// Get one line
		if( get_line(socket_number, bin, sizeof(bin)) <= 0 )
		{
// If we received en empty line before we receive a track, return with an error.
			if( (*cd)->track[0] == NULL )
			{
				perror("Error while receiving track information!");
				return(-5);
			}
			else
				break;
		}

		if( strncmp(bin, "TTITLE", 6) == 0 )
		{
			CDTrack *receivedtrack = calloc(1, sizeof(CDTrack));
			CDTrack *curtrack = NULL;
			
// the line has the format: "TTITLEnn=<title>"
// so we save anything after the "=" as tracktitle, and the number between "TTITLE" and the "=" as track.
			strncpy (receivedtrack->number, bin + 6, 10);
			if( strchr (receivedtrack->number, '=') != NULL )
				*strchr (receivedtrack->number, '=') = '\0';
// Make the number twodigit
			if ( atoi (receivedtrack->number) + 1 > 9 )
				snprintf (receivedtrack->number, 19, "%i", atoi (receivedtrack->number) + 1);
			else
				snprintf (receivedtrack->number, 19, "0%i", atoi (receivedtrack->number) + 1);
			
			if( strchr(bin, '=') != NULL )
				strncpy (receivedtrack->title, strchr(bin, '=') + 1, 2048);
			
// it is possible to split a track name via multiple freedb lines.
// thus, the same track may have occured before. In that case, we just append the current line to the older one.
			curtrack = (CDTrack *)(*cd)->track[atoi(receivedtrack->number) - 1];
			if( curtrack != NULL )
			{
// we must be aware of the maxlen of 2048.
				strncpy(curtrack->title + strlen(curtrack->title), receivedtrack->title, 2048 - strlen(curtrack->title) );
				free(receivedtrack);
			}
			else
			{
				*(receivedtrack->filename) = '\0';
				((CDTrack *)(*cd)->track[atoi(receivedtrack->number) - 1]) = receivedtrack;
				(*cd)->track[atoi(receivedtrack->number)] = NULL;
			}
		}

		if( strncmp(bin, "DYEAR", 5) == 0 )
		{
// the line has the format: "DYEAR=<title>"
// so we save anything after the "=" as year.
			if( strchr(bin, '=') != NULL )
				strncpy((*cd)->year, strchr(bin, '=') + 1, 4);
		}
	}

// Close the socket connection with my host
	return 0;
}










gint
freedb_get_albumlist_by_searchstring(gint socket_number, gchar *pstring, GList **cdlist)
{
	gchar bin[4096] = "";		// Binary in data
	gchar bout[4096] = "";		// Binary out data
	gchar buf[4096] = "";
	gchar searchstring[4096] = "";
	gchar hostname[1024] = "www.freedb.org";
	gint host_port = 80;
	gchar *temp = NULL;
	
	gboolean have = FALSE;
	gint i = 0;
	CDInfo *cd = NULL;

// A searchsting is mandtory.
	if( !pstring
		|| *pstring == '\0' )
		return 0;
	
// A socket > 0 is mandatory.
	if( socket_number <= 0 )
		return(-1);
	
// Change all spaces in the searchstring to "+" (URLsafe)
	strncpy(searchstring, pstring, sizeof(searchstring));
	while( strchr(searchstring, ' ') != NULL )
		*strchr(searchstring, ' ') = '+';
	
// Prepare request
	sprintf(bout, "GET http://%s/freedb_search.php?words=%s&allfields=YES&fields=artist&fields=title&allcats=YES&grouping=none HTTP/1.1\nConnection: close\nHost: %s:%i\n\n",
		hostname,
		searchstring,
		hostname,
		host_port);
	
// Send request
	if( write(socket_number, bout, strlen(bout)) < 0 )
	{
		perror("Error while sending request to the server!\n");
		return -4;
	}

// Ok! Now I read the answer
	i = 0;
	have = FALSE;
	while( strncmp(bin, "</body>", 7) != 0 )
	{
// Get one line
		if( get_line(socket_number, bin, sizeof(bin)) <= 0 )
		{
			g_error("Error while receiving cd information! (have = %i)\n", have);
			return -7;
		}
		
		if( strncmp(bin, "<tr><td><a href=\"http://www.freedb.org/freedb_search_fmt.php?cat=", 65) == 0 )
		{
// ok, we have found one line that looks like a cd title in the html output.
			cd = calloc (1, sizeof(CDInfo));
			
// the cd title should begin after char nr. 64 and end at the first occurence of "</a><br><"
// so we strip all unneeded chars at the string end here.
			temp = &bin[64];
			while( *(++temp) != '\0'
				&& *temp != '\n' )
			{
				if( strncmp(temp, "</a><br><", 9) == 0 )
				{
					*temp = '\0';
					break;
				}
			}
			
// maxlen 1024.
			strncpy(cd->cdtitle, strchr(&bin[65], '>') + 1, 1023);

// the genre begins at char 65
			strncpy(cd->genre, &bin[65], 511);
			if( strchr(cd->genre, '&') )
				*strchr(cd->genre, '&') = '\0';
			else
			{
				perror("Wrong format of the freedb output! Contact this programs author if this happens often.");
				free(cd);
				return -8;
			}

// the freedb id begins after the first "=" after char nr. 65
			if( strchr(&bin[65], '=')
				&& strchr(strchr(&bin[65], '='), '\"') )
			{
				strncpy(buf, strchr(&bin[65], '=')+1, sizeof(buf));
				*strchr(buf, '\"') = '\0';
				cd->id = strtoul(buf, NULL, 16);
			}
			else
			{
				perror("Wrong format of the freedb output! Contact this programs author if this happens often.");
				free(cd);
				return -9;
			}
			
			cd->track[0] = NULL;
			*cdlist = g_list_append(*cdlist, cd);
			have = TRUE;
		}
		else if( strncmp(bin, "<br><br></tr>", 13) != 0
				&& have == TRUE )
		{
			break;
		}
	}

// Close the socket connection with my host
	return 0;
}
